/**
 * COOS - Connected Objects Operating System (www.connectedobjects.org).
 *
 * Copyright (C) 2009 Telenor ASA and Tellu AS. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may also contact one of the following for additional information:
 * Telenor ASA, Snaroyveien 30, N-1331 Fornebu, Norway (www.telenor.no)
 * Tellu AS, Hagalokkveien 13, N-1383 Asker, Norway (www.tellu.no)
 */
package org.coos.plugin.logserver.impl;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import org.coos.actorframe.ActorSM;
import org.coos.messaging.util.Log;
import org.coos.messaging.util.LogFactory;

public class LogServerSM extends ActorSM {
	static Log LOG = LogFactory.getLog(LogServerSM.class);
	public static final String CONFIG_FILE_KEY = "logback.configurationFile";
	LogbackServerNode logbackServerNode;

	public LogServerSM() {
		setBehaviorClass(new LogServerCS("LogServer"));
	}

	protected void initInstance() {
		super.initInstance();
		setVisible(true);
	}

	/**
	 * Configure the Logback Server.
	 * 
	 * @param configuration
	 *            name of configuration file/resource
	 * @throws IOException
	 *             if configuration not found or could not be read
	 * @throws Exception
	 *             if parsing or unknown exception
	 */
	protected void configure(String configuration) throws IOException, Exception {
		InputStream is = getResource(configuration);
		logbackServerNode = new LogbackServerNode(this, is);

	}

	protected void processLoggingEvent(byte[] serializedEvent) {
		if (logbackServerNode == null) {
			LOG.warn("Using existing logback configuration.");
			logbackServerNode = new LogbackServerNode(this);
		}
		logbackServerNode.logTheEvent(serializedEvent);
	}

	private InputStream getResource(String resourceName) throws IOException {
		InputStream is = null;
		String baseName;

		if (resourceName.isEmpty()) {
			return null;
		}

		if (resourceName.startsWith("/")) {
			baseName = resourceName.substring(1);
		} else {
			baseName = resourceName;
			resourceName = "/" + resourceName;
		}

		try {
			File file = new File(".", baseName);
			URL url = file.toURI().toURL();
			is = url.openStream();
			if (is != null) {
				LOG.info("Found configuration " + url);
				return is;
			}
		} catch (Exception ignore) {
		}

		try {
			File file = new File(resourceName);
			URL url = file.toURI().toURL();
			is = url.openStream();
			if (is != null) {
				LOG.info("Found configuration " + url);
				return is;
			}
		} catch (Exception ignore) {
		}

		try {
			is = this.getClass().getResourceAsStream(resourceName);
			if (is != null) {
				LOG.info("Found configuration as resource " + resourceName);
				return is;
			}
		} catch (Exception ignore) {
		}

		throw new IOException("Cannot find " + resourceName);
	}
}
